const { MessengerContext } = require('toolbot-core-experiment');
const { MessengerClient } = require('messaging-api-messenger');

const FacebookEvent = require('./FacebookEvent');

module.exports = class FacebookConnector {
  constructor({ accessToken }) {
    this._client = MessengerClient.connect(accessToken);

    Object.defineProperty(this._client, 'sendPrivateReply', {
      enumerable: false,
      configurable: true,
      writable: true,
      value(commentId, message) {
        const http = this.getHTTPClient();
        return http.post(
          `/${commentId}/private_replies?access_token=${this._accessToken}`,
          {
            message,
          }
        );
      },
    });

    // https://developers.facebook.com/docs/graph-api/reference/v2.10/object/comments/
    Object.defineProperty(this._client, 'sendComment', {
      enumerable: false,
      configurable: true,
      writable: true,
      value(objectId, message) {
        const http = this.getHTTPClient();
        return http.post(
          `/${objectId}/comments?access_token=${this._accessToken}`,
          {
            message,
          }
        );
      },
    });
  }

  _getRawEventsFromRequest(body) {
    if (body.entry) {
      const { entry } = body;

      return entry
        .map(ent => {
          if (ent.messaging) {
            return ent.messaging[0];
          }

          if (ent.standby) {
            return ent.standby[0];
          }

          // Patch For Facebook Start
          if (ent.changes) {
            return ent.changes[0];
          }
          // Patch For Facebook End

          return null;
        })
        .filter(event => event != null);
    }

    return [body];
  }

  _isStandby(body) {
    if (!body.entry) return false;
    const entry = body.entry[0];

    return !!entry.standby;
  }

  // FIXME: facebook? but should enable verifyToken in createServer
  get platform() {
    return 'messenger';
  }

  get client() {
    return this._client;
  }

  getUniqueSessionIdFromRequest(body) {
    const rawEvent = this._getRawEventsFromRequest(body)[0];
    if (rawEvent.message && rawEvent.message.is_echo && rawEvent.recipient) {
      return rawEvent.recipient.id;
    }
    if (rawEvent.sender) {
      return rawEvent.sender.id;
    }
    return null;
  }

  async updateSession(session, body) {
    if (!session.user) {
      const senderId = this.getUniqueSessionIdFromRequest(body);
      const user = await this._client.getUserProfile(senderId);

      session.user = {
        id: senderId,
        ...user,
      };
    }
  }

  mapRequestToEvents(body) {
    const rawEvents = this._getRawEventsFromRequest(body);
    const isStandby = this._isStandby(body);
    return rawEvents.map(event => new FacebookEvent(event, { isStandby }));
  }

  createContext({ event, session }) {
    return new MessengerContext({
      client: this._client,
      event,
      session,
    });
  }
};
